#!/usr/bin/env python
import os
import sys

import django
from django.conf import settings
from django.test.utils import get_runner

def runtests():
	os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.test_settings'
	django.setup()
	TestRunner = get_runner(settings)
	test_runner = TestRunner(verbosity=2)
	failures = test_runner.run_tests(["tests.__init__"])
	sys.exit(bool(failures))

if __name__ == "__main__":
	runtests()



from collectd_rest.rrd import render, RRDError
from django.db import models
from django.core.exceptions import ValidationError

class GraphGroup(models.Model):
	name = models.CharField(max_length=256, blank = False, unique = True)
	title = models.CharField(max_length=256, blank = False)

class Graph(models.Model):
	name = models.CharField(max_length=256, blank = False)
	title = models.CharField(max_length=256, blank = True)
	priority = models.IntegerField(default=0)
	group = models.ForeignKey('GraphGroup', related_name='graphs')
	command = models.TextField(blank = False)

	class Meta:
		unique_together = ("name", "group")
		ordering = ['-priority']

	def clean(self):
		try:
			render(self.command, 'PNG')
		except RRDError as e:
			raise ValidationError({'command':e.message})




import shlex, subprocess

class RRDError(Exception):
	def __init__(self, message):
		self.message = message

def render(command, format):
	args = ['rrdtool','graph','-','--imgformat',format] + shlex.split(command)
	p = subprocess.Popen(args, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	code = p.wait()
	if code != 0:
		raise RRDError(p.stderr.read())
	return p.stdout.read()



from collectd_rest import models
from rest_framework import serializers

class GraphSerializer(serializers.ModelSerializer):
	group = serializers.SlugRelatedField(slug_field='name', queryset=models.GraphGroup.objects.all(), required=True)
	url = serializers.HyperlinkedIdentityField(view_name='graph-detail', read_only=True)

	class Meta:
		model = models.Graph
		fields = ('id', 'name', 'title', 'group', 'url', 'command', 'priority')

	def validate(self, attrs):
		instance = GraphSerializer.Meta.model(**attrs)
		instance.clean()
		return attrs

class GraphGroupSerializer(serializers.ModelSerializer):
	graphs = GraphSerializer(many=True, read_only=True)

	class Meta:
		model = models.GraphGroup
		fields = ('id', 'name', 'title', 'graphs')



from rest_framework import renderers
from collectd_rest.rrd import render

class PNGRenderer(renderers.BaseRenderer):
	media_type = 'image/png'
	format = 'png'
	charset = None
	render_style = 'binary'
	
	def render(self, data, media_type=None, renderer_context=None):
		command = data['command']
		return render(command, 'PNG')



from collectd_rest import views
from django.conf.urls import url, include
from rest_framework.routers import DefaultRouter
from rest_framework.urlpatterns import format_suffix_patterns

router = DefaultRouter()
router.register(r'groups', views.GraphGroupViewSet)
router.register(r'graphs', views.GraphViewSet)

urlpatterns = [
	url(r'^', include(router.urls)),
]
